/**
 * <p>A container that stores elements, where access is only given to
 * the "top" element in the container. New elements are added to the
 * container by <em>pushing</em> the items to the top of the
 * stack. Items are removed by <em>popping</em> them from the top. The
 * order of element access is "last in, first out" (LIFO): the item at
 * the top of the stack (and the item that a pop will remove) is the
 * most recently pushed item that is contained in the stack.</p>
 *
 * <p>The state of a stack can be represented by the sequence of
 * elements contained in the stack. For example, the state of a stack
 * containing <code>n</code> elements can be written as sequence
 * <code>x_1 x_2 ... x_{n-1} x_n</code>, where <code>x_n</code> is at
 * the top of the stack. From this state, pushing the element
 * <code>y</code> onto the stack will result in a state <code>x_1 x_2
 * ... x_{n-1} x_n y</code> (containing <code>n+1</code> elements),
 * while popping the stack will return the value <code>x_n</code> and
 * result in the state <code>x_1 x_2 ... x_{n-1}</code>.</p>
 *
 */

public interface SimpleStack<E> {
    /**
     * Get the current size of the stack (i.e., the number of elements
     * currently stored in the stack).
     *
     * @return the current size
     */
    int size();

    /**
     * Determine if the stack is currently empty (i.e., currently
     * contains no elements).
     *
     * @return true if and only if the stack is empty
     */
    boolean isEmpty();

    /**
     * <p>Push a new element <code>x</code> onto the stack. After this
     * operation, the element <code>x</code> will be at the top of the
     * stack.</p>
     *
     * @param x the element to be pushed onto the stack
     */
    void push(E x);

    /**
     * <p>Remove and return the top element from the stack, if the
     * stack is not empty. If the stack was previously in the state
     * <code>x_1 x_2 ... x_{n-1} x_n</code> with <code>x_n</code> at
     * the top, the resulting state will be <code>x_1 x_2
     * ... x_{n-1}</code> and the element <code>x_n</code> will be
     * returned. If the stack is empty, an
     * <code>EmptyStackException</code> is thrown.</p>
     *
     * @return the top element on the stack
     * @throws java.util.EmptyStackException if the stack is empty
     */
    E pop();

    /**
     * <p>Return the top element on the stack, or <code>null</code> if
     * the stack is empty. The state of the stack is unchanged after
     * this operation.</p>
     *
     * @return the top element on the stack, or if empty,
     * <code>null</code>
     */
    E peek();
}
